@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2024 Intel Corporation.
+ *   Copyright(c) 2016-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -55,8 +55,12 @@
 /* Defines */
 #ifdef QAT_HW
 # ifdef QAT_HW_INTREE
-#  define ENABLE_QAT_HW_SHA3
-#  define ENABLE_QAT_HW_CHACHAPOLY
+#  ifndef ENABLE_QAT_HW_SHA3
+#   define ENABLE_QAT_HW_SHA3
+#  endif
+#  ifndef ENABLE_QAT_HW_CHACHAPOLY
+#   define ENABLE_QAT_HW_CHACHAPOLY
+#  endif
 # endif
 #endif
 
@@ -147,6 +151,7 @@
 # define VAES_BIT 9
 # define VPCLMULQDQ_BIT 10
 # define AVX512F_BIT 16
+# define AVX2_BIT 5
 #endif
 
 #define QAT_MAX_INPUT_STRING_LENGTH 1024
@@ -164,23 +169,23 @@ int qat_fips_kat_test;
 const char *engine_qat_id = STR(QAT_ENGINE_ID);
 #if defined(QAT_HW) && defined(QAT_SW)
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v1.6.1";
+    "Reference implementation of QAT crypto engine(qat_hw & qat_sw) v1.8.1";
 #elif QAT_HW
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_hw) v1.6.1";
+    "Reference implementation of QAT crypto engine(qat_hw) v1.8.1";
 #else
 const char *engine_qat_name =
-    "Reference implementation of QAT crypto engine(qat_sw) v1.6.1";
+    "Reference implementation of QAT crypto engine(qat_sw) v1.8.1";
 #endif
 unsigned int engine_inited = 0;
 int fallback_to_openssl = 0;
 #if defined(QAT_OPENSSL_3) && !defined(QAT_OPENSSL_PROVIDER)
 int qat_openssl3_prf_fallback = 0;
 int qat_openssl3_hkdf_fallback = 0;
-int qat_openssl3_sm2_fallback = 0;
-int qat_openssl3_sm3_fallback = 0;
 int qat_openssl3_sha_fallback = 0;
 #endif
+int qat_openssl3_sm2_fallback = 0;
+int qat_openssl3_sm3_fallback = 0;
 int fallback_to_qat_sw = 0; /* QAT HW initialize fail, offload to QAT SW. */
 int qat_hw_offload = 0;
 int qat_sw_offload = 0;
@@ -340,7 +345,13 @@ __thread unsigned long long num_ecx_sw_derive_reqs = 0;
 __thread unsigned long long num_sm4_cbc_hw_cipher_reqs = 0;
 __thread unsigned long long num_sm4_cbc_sw_cipher_reqs = 0;
 
-#ifndef QAT_BORINGSSL
+#ifndef __FreeBSD__
+clock_t clock_id = CLOCK_MONOTONIC_RAW;
+#else
+clock_t clock_id = CLOCK_MONOTONIC_PRECISE;
+#endif
+
+#if ! defined(QAT_BORINGSSL) && ! defined(QAT_OPENSSL_PROVIDER)
 const ENGINE_CMD_DEFN qat_cmd_defns[] = {
     {
         QAT_CMD_ENABLE_EXTERNAL_POLLING,
@@ -434,7 +445,6 @@ const ENGINE_CMD_DEFN qat_cmd_defns[] = {
         "SET_CONFIGURATION_SECTION_NAME",
         "Set the configuration section to use in QAT driver configuration file",
         ENGINE_CMD_FLAG_STRING},
-# ifndef __FreeBSD__
     {
         QAT_CMD_ENABLE_SW_FALLBACK,
         "ENABLE_SW_FALLBACK",
@@ -445,7 +455,6 @@ const ENGINE_CMD_DEFN qat_cmd_defns[] = {
         "HEARTBEAT_POLL",
         "Check the acceleration devices are still functioning",
         ENGINE_CMD_FLAG_NO_INPUT},
-# endif
     {
         QAT_CMD_DISABLE_QAT_OFFLOAD,
         "DISABLE_QAT_OFFLOAD",
@@ -546,6 +555,7 @@ int qat_sw_cpu_support(void)
     __cpuid(info, 0x07, 0);
 
     unsigned int avx512f = 0;
+    unsigned int avx2 = 0;
     unsigned int vaes    = 0;
     unsigned int vpclmulqdq = 0;
 
@@ -558,14 +568,16 @@ int qat_sw_cpu_support(void)
     if (*ecx & (0x1 << VPCLMULQDQ_BIT))
         vpclmulqdq = 1;
 
-    DEBUG("QAT_SW - Processor supported: AVX512F = %u, VAES = %u, VPCLMULQDQ = %u\n",
-           avx512f, vaes, vpclmulqdq);
+    if (*ebx & (0x1 << AVX2_BIT))
+        avx2 = 1;
 
-    if (avx512f && vaes && vpclmulqdq) {
-        return 1;
+    if ((avx512f || avx2) && vaes && vpclmulqdq) {
+         DEBUG("QAT_SW - Processor supported: AVX512F = %u, AVX2 = %u, VAES = %u, VPCLMULQDQ = %u\n",
+               avx512f, avx2, vaes, vpclmulqdq);
+	 return 1;
     } else {
-        fprintf(stderr, "QAT_SW - Processor unsupported: AVX512F = %u, VAES = %u, VPCLMULQDQ = %u\n",
-                avx512f, vaes, vpclmulqdq);
+        fprintf(stderr, "\nQAT_SW - Processor unsupported: AVX512F = %u, AVX2 = %u, VAES = %u, VPCLMULQDQ = %u\n",
+                avx512f, avx2, vaes, vpclmulqdq);
         return 0;
     }
 }
@@ -739,6 +751,7 @@ int qat_engine_finish(ENGINE *e)
  *         ENGINE_init
  ******************************************************************************/
 
+#ifndef QAT_OPENSSL_PROVIDER
 int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
 {
     unsigned int retVal = 1;
@@ -970,20 +983,14 @@ int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
         }
         break;
         case QAT_CMD_ENABLE_SW_FALLBACK:
-# if !defined(__FreeBSD__) && !defined(QAT_HW_INTREE)
         DEBUG("Enabled SW Fallback\n");
         BREAK_IF(engine_inited, \
                 "ENABLE_SW_FALLBACK failed as the engine is already initialized\n");
         enable_sw_fallback = 1;
         CRYPTO_QAT_LOG("SW Fallback enabled - %s\n", __func__);
-# else
-        WARN("QAT_CMD_ENABLE_SW_FALLBACK is not supported\n");
-        retVal = 0;
-# endif
         break;
 
         case QAT_CMD_HEARTBEAT_POLL:
-# if !defined(__FreeBSD__) && !defined(QAT_HW_INTREE)
         BREAK_IF(!engine_inited, "HEARTBEAT_POLL failed as engine is not initialized\n");
         BREAK_IF(!enable_external_polling,
                 "HEARTBEAT_POLL failed as external polling is not enabled\n");
@@ -996,10 +1003,6 @@ int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
             WARN("HEARTBEAT_POLL failed as no instances are available\n");
             retVal = 0;
         }
-# else
-        WARN("QAT_CMD_HEARTBEAT_POLL is not supported\n");
-        retVal = 0;
-# endif
         break;
 
         case QAT_CMD_DISABLE_QAT_OFFLOAD:
@@ -1054,6 +1057,7 @@ int qat_engine_ctrl(ENGINE *e, int cmd, long i, void *p, void (*f) (void))
     }
     return retVal;
 }
+#endif /* QAT_OPENSSL_PROVIDER */
 
 #ifdef ENABLE_QAT_HW_KPT
 EVP_PKEY *qat_engine_load_privkey(ENGINE *e, const char *key_id, UI_METHOD *ui_method, void *callback_data)
@@ -1113,7 +1117,7 @@ int bind_qat(ENGINE *e, const char *id)
    int ret = 0;
 #ifdef QAT_HW
     char *config_section = NULL;
-# if defined(QAT20_OOT) || defined(__FreeBSD__)
+# if !defined(QAT_HW_INTREE) && (defined(QAT20_OOT) || defined(__FreeBSD__))
     Cpa32U dev_count = 0;
 # endif
 #endif
@@ -1334,6 +1338,15 @@ int bind_qat(ENGINE *e, const char *id)
         }
 # endif
 
+# ifdef ENABLE_QAT_SW_SM2
+    if (!qat_hw_sm2_offload &&
+        (qat_sw_algo_enable_mask & ALGO_ENABLE_MASK_SM2) &&
+         mbx_get_algo_info(MBX_ALGO_EC_SM2)) {
+         qat_sw_sm2_offload = 1;
+         INFO("QAT_SW SM2 for Provider Enabled\n");
+    }
+# endif
+
 # ifdef ENABLE_QAT_SW_GCM
         qat_sw_gcm_offload = 1;
         DEBUG("QAT_SW GCM for Provider Enabled\n");
@@ -1351,6 +1364,9 @@ int bind_qat(ENGINE *e, const char *id)
     /* Create static structures for ciphers now
      * as this function will be called by a single thread. */
     qat_create_ciphers();
+    /* Initialize EVP_MD methods for supported digest algorithms.
+     * This sets up the digest methods for both hardware and software digests. */
+    qat_create_digest_meth();
 # ifndef QAT_DEBUG
     if (qat_sw_gcm_offload && !qat_hw_gcm_offload)
         INFO("QAT_SW GCM for Provider Enabled\n");
@@ -1375,11 +1391,12 @@ int bind_qat(ENGINE *e, const char *id)
         INFO("QAT_HW SM4-CBC for Provider Enabled\n");
 # endif
 # ifdef ENABLE_QAT_SW_SM3
-        qat_sw_sm3_offload = 1;
+    if (qat_sw_sm3_offload) {
         INFO("QAT_SW SM3 for Provider Enabled\n");
+    }
 # endif
 # ifdef ENABLE_QAT_HW_CCM
-        qat_hw_aes_ccm_offload = 1;
+    if (qat_hw_aes_ccm_offload)
         INFO("QAT_HW AES-CCM for Provider Enabled\n");
 # endif
 #endif
@@ -1419,6 +1436,7 @@ int bind_qat(ENGINE *e, const char *id)
     return ret;
 }
 
+#ifndef QAT_OPENSSL_PROVIDER
 #ifndef OPENSSL_NO_DYNAMIC_ENGINE
 IMPLEMENT_DYNAMIC_BIND_FN(bind_qat)
 IMPLEMENT_DYNAMIC_CHECK_FN()
@@ -1501,4 +1519,5 @@ void ENGINE_unload_qat(void)
     }
 }
 #endif /* QAT_BORINGSSL */
+#endif /* QAT_OPENSSL_PROVIDER */
 #endif
