@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2016-2024 Intel Corporation.
+ *   Copyright(c) 2016-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -46,46 +46,37 @@
 #ifndef E_QAT_H
 # define E_QAT_H
 
-# include <openssl/engine.h>
+# ifndef QAT_OPENSSL_PROVIDER
+#  include <openssl/engine.h>
+# endif
+# include <openssl/ec.h>
+# include <openssl/err.h>
 # include <sys/types.h>
 # include <unistd.h>
 # include <string.h>
 # include <semaphore.h>
 # include <sched.h>
 
-#ifndef QAT_BORINGSSL
-# include <openssl/async.h>
-#endif
-
-#ifdef QAT_BORINGSSL
-# include "qat_bssl.h"
-#endif /* QAT_BORINGSSL */
-
-# ifdef QAT_OPENSSL_3
-#  include "qat_prov_err.h"
-# elif defined(QAT_BORINGSSL)
+# ifdef QAT_BORINGSSL
+#  include "qat_bssl.h"
 #  include "qat_bssl_err.h"
 # else
-#  include "e_qat_err.h"
-# endif
+#  include "qat_err.h"
+#  include <openssl/async.h>
+#endif /* QAT_BORINGSSL */
 
 # ifdef QAT_HW
 #  include "cpa.h"
 #  include "cpa_types.h"
 #  include "cpa_cy_common.h"
-#  ifdef USE_QAT_CONTIG_MEM
-#   include "qae_mem_utils.h"
-#  endif
-#  ifdef USE_USDM_MEM
-#   include "qat_hw_usdm_inf.h"
-#  endif
+#  include "qat_hw_usdm_inf.h"
 # endif
 
 # ifdef QAT_SW
 #  include "qat_sw_queue.h"
 #  include "qat_sw_freelist.h"
 # endif
-# include "qat_common.h"
+#include "qat_common.h"
 
 # ifndef ERR_R_RETRY
 #  define ERR_R_RETRY 57
@@ -128,6 +119,7 @@
 # define QAT_ATOMIC_DEC(qat_int)              \
             (__sync_sub_and_fetch(&(qat_int), 1))
 
+extern clock_t clock_id;
 # ifdef QAT_HW
 typedef struct {
     int qatAsymInstanceNumForThread;
@@ -404,10 +396,10 @@ extern int fallback_to_openssl;
 #if defined(QAT_OPENSSL_3) && !defined(QAT_OPENSSL_PROVIDER)
 extern int qat_openssl3_prf_fallback;
 extern int qat_openssl3_hkdf_fallback;
-extern int qat_openssl3_sm2_fallback;
-extern int qat_openssl3_sm3_fallback;
 extern int qat_openssl3_sha_fallback;
 #endif
+extern int qat_openssl3_sm3_fallback;
+extern int qat_openssl3_sm2_fallback;
 extern int fallback_to_qat_sw; /* QAT HW initialization fail, offload to QAT SW. */
 extern int qat_hw_offload;
 extern int qat_sw_offload;
@@ -931,7 +923,8 @@ int qat_sw_cpu_support(void);
 # endif
 
 # ifdef QAT_OPENSSL_PROVIDER
-typedef _Atomic int CRYPTO_REF_COUNT;
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+
 static __inline__ int CRYPTO_UP_REF(int *val, int *ret, ossl_unused void *lock)
 {
     *ret = __atomic_fetch_add(val, 1, __ATOMIC_RELAXED) + 1;
@@ -946,16 +939,16 @@ static __inline__ int CRYPTO_DOWN_REF(int *val, int *ret,
 	__atomic_thread_fence(__ATOMIC_ACQUIRE);
     return 1;
 }
-# endif
 
-# if OPENSSL_VERSION_NUMBER >= 0x30200000
-static __inline__ int QAT_CRYPTO_UP_REF(QAT_CRYPTO_REF_COUNT *refcnt, int *ret)
+# else
+
+static __inline__ int QAT_CRYPTO_UP_REF(CRYPTO_REF_COUNT *refcnt, int *ret)
 {
     *ret = __atomic_fetch_add(&refcnt->val, 1, __ATOMIC_RELAXED) + 1;
     return 1;
 }
 
-static __inline__ int QAT_CRYPTO_DOWN_REF(QAT_CRYPTO_REF_COUNT *refcnt, int *ret)
+static __inline__ int QAT_CRYPTO_DOWN_REF(CRYPTO_REF_COUNT *refcnt, int *ret)
 {
     *ret = __atomic_fetch_sub(&refcnt->val, 1, __ATOMIC_RELAXED) - 1;
     if (*ret == 0)
@@ -963,5 +956,6 @@ static __inline__ int QAT_CRYPTO_DOWN_REF(QAT_CRYPTO_REF_COUNT *refcnt, int *ret
     return 1;
 }
 # endif
+# endif
 
 #endif   /* E_QAT_H */
