@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2019-2024 Intel Corporation.
+ *   Copyright(c) 2019-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,6 @@
 # define QAT_EVP_H
 
 
-# include <openssl/engine.h>
 # include <openssl/ossl_typ.h>
 # include "e_qat.h"
 
@@ -110,20 +109,32 @@ extern const EVP_PKEY_METHOD *sw_x25519_pmeth;
 extern const EVP_PKEY_METHOD *sw_x448_pmeth;
 
 # ifdef ENABLE_QAT_HW_ECX
+#  ifndef QAT_OPENSSL_PROVIDER
 int qat_pkey_ecx25519_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey);
 int qat_pkey_ecx448_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey);
 int qat_pkey_ecx_derive25519(EVP_PKEY_CTX *ctx, unsigned char *key, size_t *keylen);
 int qat_pkey_ecx_derive448(EVP_PKEY_CTX *ctx, unsigned char *key, size_t *keylen);
-#  ifndef QAT_OPENSSL_PROVIDER
 int qat_pkey_ecx_ctrl(EVP_PKEY_CTX *ctx, int type, int p1, void *p2);
+#  else
+void *qat_pkey_ecx25519_keygen(void *ctx, OSSL_CALLBACK *osslcb, void *cbarg);
+void *qat_pkey_ecx448_keygen(void *ctx, OSSL_CALLBACK *osslcb, void *cbarg);
+int qat_pkey_ecx_derive25519(void *ctx, unsigned char *key, size_t *keylen,
+                             size_t outlen);
+int qat_pkey_ecx_derive448(void *ctx, unsigned char *key, size_t *keylen,
+                           size_t outlen);
 #  endif
 # endif
 
 # ifdef ENABLE_QAT_SW_ECX
+#  ifndef QAT_OPENSSL_PROVIDER
 int multibuff_x25519_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey);
 int multibuff_x25519_derive(EVP_PKEY_CTX *ctx, unsigned char *key, size_t *keylen);
-#  ifndef QAT_OPENSSL_PROVIDER
 int multibuff_x25519_ctrl(EVP_PKEY_CTX *ctx, int type, int p1, void *p2);
+#  else
+void* multibuff_x25519_keygen(void *ctx, OSSL_CALLBACK *osslcb,
+                              void *cbarg);
+int multibuff_x25519_derive(void *ctx, unsigned char *key,
+                            size_t *keylen, size_t outlen);
 #  endif
 # endif
 
@@ -162,6 +173,7 @@ int qat_ciphers(ENGINE *e, const EVP_CIPHER **cipher, const int **nids,
                 int nid);
 const EVP_CIPHER *qat_create_gcm_cipher_meth(int nid, int keylen);
 const EVP_CIPHER *qat_create_ccm_cipher_meth(int nid, int keylen);
+const EVP_CIPHER *qat_gcm_cipher_sw_impl(int nid);
 # ifndef ENABLE_QAT_SMALL_PKT_OFFLOAD
 #  define CRYPTO_SMALL_PACKET_OFFLOAD_THRESHOLD_DEFAULT 2048
 #  define CRYPTO_SMALL_PACKET_OFFLOAD_THRESHOLD_SM4_CBC 64
