@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2019-2024 Intel Corporation.
+ *   Copyright(c) 2019-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -51,10 +51,11 @@
 #include <signal.h>
 #include <stdarg.h>
 
-#include "openssl/ossl_typ.h"
-#include "openssl/kdf.h"
-#include "openssl/evp.h"
-#include "openssl/ssl.h"
+#include <openssl/ossl_typ.h>
+#include <openssl/kdf.h>
+#include <openssl/evp.h>
+#include <openssl/rand.h>
+#include <openssl/ssl.h>
 #include "e_qat.h"
 #include "qat_utils.h"
 #include "qat_hw_asym_common.h"
@@ -66,17 +67,9 @@
 #else
 # include "qat_evp.h"
 #endif
-
 #ifdef ENABLE_QAT_SW_ECX
 # include "qat_sw_ecx.h"
 #endif
-#ifdef USE_QAT_CONTIG_MEM
-# include "qae_mem_utils.h"
-#endif
-#ifdef USE_USDM_MEM
-# include "qat_hw_usdm_inf.h"
-#endif
-
 #include "cpa.h"
 #include "cpa_types.h"
 #include "cpa_cy_key.h"
@@ -221,13 +214,16 @@ static int qat_pkey_ecx_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey, int type)
      * generated by qatengine, we must update its reference
      * count accordingly, otherwise it will trigger openssl's panic.
      */
-#ifdef QAT_OPENSSL_3
+#if defined(QAT_OPENSSL_3) && defined(QAT_OPENSSL_PROVIDER)
 # if OPENSSL_VERSION_NUMBER < 0x30200000
     key->references = 1;
     key->lock = CRYPTO_THREAD_lock_new();
 # else
     key->references.val = 1;
 # endif
+# ifdef QAT_OPENSSL_PROVIDER
+    key->type = gctx->type;
+# endif
 #endif
 
 #ifndef QAT_OPENSSL_PROVIDER
@@ -271,12 +267,22 @@ static int qat_pkey_ecx_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey, int type)
     default:
         WARN("Unsupported NID: %d\n", gctx->type);
         QATerr(QAT_F_QAT_PKEY_ECX_KEYGEN, ERR_R_INTERNAL_ERROR);
+#ifdef QAT_OPENSSL_3
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+        CRYPTO_THREAD_lock_free(key->lock);
+# endif
+#endif
         OPENSSL_free(key);
         return 0;
     }
 
     if (qat_get_qat_offload_disabled()) {
         DEBUG("- Switched to software mode.\n");
+#ifdef QAT_OPENSSL_3
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+        CRYPTO_THREAD_lock_free(key->lock);
+# endif
+#endif
         OPENSSL_free(key);
 
         if (!is_ecx_448) {
@@ -299,6 +305,11 @@ static int qat_pkey_ecx_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey, int type)
             goto err;
         } else {
             QATerr(QAT_F_QAT_PKEY_ECX_KEYGEN, ERR_R_INTERNAL_ERROR);
+#if defined(QAT_OPENSSL_3) && defined(QAT_OPENSSL_PROVIDER)
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+            CRYPTO_THREAD_lock_free(key->lock);
+# endif
+#endif
             OPENSSL_free(key);
             return 0;
         }
@@ -312,6 +323,11 @@ static int qat_pkey_ecx_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey, int type)
     if (NULL == qat_ecx_op_data) {
         WARN("Failed to allocate memory for qat_ecx_op_data\n");
         QATerr(QAT_F_QAT_PKEY_ECX_KEYGEN, ERR_R_MALLOC_FAILURE);
+#if defined(QAT_OPENSSL_3) && defined(QAT_OPENSSL_PROVIDER)
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+        CRYPTO_THREAD_lock_free(key->lock);
+# endif
+#endif
         OPENSSL_free(key);
         return 0;
     }
@@ -537,6 +553,10 @@ static int qat_pkey_ecx_keygen(EVP_PKEY_CTX *ctx, EVP_PKEY *pkey, int type)
         QATerr(QAT_F_QAT_PKEY_ECX_KEYGEN, ERR_R_INTERNAL_ERROR);
         goto err;
     }
+
+#ifdef QAT_OPENSSL_3
+    key->haspubkey = 1;
+#endif
 #ifdef QAT_OPENSSL_PROVIDER
     qat_pkey_ecx_keygen_op_data_free(pXk, qat_ecx_op_data, qat_keylen, qat_svm);
     return key;
@@ -554,6 +574,11 @@ err:
             privkey = NULL;
         }
         if (NULL != key) {
+#if defined(QAT_OPENSSL_3) && defined(QAT_OPENSSL_PROVIDER)
+# if OPENSSL_VERSION_NUMBER < 0x30200000
+            CRYPTO_THREAD_lock_free(key->lock);
+# endif
+#endif
             OPENSSL_free(key);
             key = NULL;
         }
