@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2020-2024 Intel Corporation.
+ *   Copyright(c) 2020-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -128,6 +128,7 @@ int qat_use_signals(void)
        been initialised then there will be a further check within
        qat_engine_init inside a mutex to prevent a race condition. */
 
+#ifndef QAT_OPENSSL_PROVIDER
     if (unlikely(!engine_inited)) {
         ENGINE* e = ENGINE_by_id(engine_qat_id);
 
@@ -146,6 +147,7 @@ int qat_use_signals(void)
         ENGINE_free(e);
         ENGINE_QAT_PTR_RESET();
     }
+#endif
 
     return qat_use_signals_no_engine_start();
 }
@@ -226,6 +228,7 @@ int get_instance(int inst_type, int mem_type)
 
     unsigned int inst_count = 0;
     thread_local_variables_t * tlv = NULL;
+#ifndef QAT_OPENSSL_PROVIDER
     /* See qat_use_signals() above for more info on why it is safe to
        check engine_inited outside of a mutex in this case. */
     if (unlikely(!engine_inited)) {
@@ -245,6 +248,7 @@ int get_instance(int inst_type, int mem_type)
         ENGINE_free(e);
         ENGINE_QAT_PTR_RESET();
     }
+#endif
 
     tlv = qat_check_create_local_variables();
     if (unlikely(NULL == tlv)) {
@@ -376,7 +380,6 @@ static void qat_local_variable_destructor(void *tlv)
 }
 
 
-#ifndef __FreeBSD__
 void qat_instance_notification_callbackFn(const CpaInstanceHandle ih, void *callbackTag,
                                           const CpaInstanceEvent inst_ev)
 {
@@ -390,7 +393,7 @@ void qat_instance_notification_callbackFn(const CpaInstanceHandle ih, void *call
             packageId =
                 qat_instance_details[(intptr_t)callbackTag].qat_instance_info.physInstId.packageId;
             qat_accel_details[packageId].qat_accel_reset_status = 1;
-            clock_gettime(CLOCK_MONOTONIC_RAW, &ts);
+            clock_gettime(clock_id, &ts);
             CRYPTO_QAT_LOG("[%lld.%06ld] Instance: %ld Handle %p Device %d RESTARTING \n",
                     (long long)ts.tv_sec, ts.tv_nsec / NANOSECONDS_TO_MICROSECONDS,
                     (intptr_t)callbackTag, ih, packageId);
@@ -405,7 +408,7 @@ void qat_instance_notification_callbackFn(const CpaInstanceHandle ih, void *call
             packageId =
                 qat_instance_details[(intptr_t)callbackTag].qat_instance_info.physInstId.packageId;
             qat_accel_details[packageId].qat_accel_reset_status = 0;
-            clock_gettime(CLOCK_MONOTONIC_RAW, &ts);
+            clock_gettime(clock_id, &ts);
             CRYPTO_QAT_LOG("[%lld.%06ld] Instance: %ld Handle %p Device %d RESTARTED \n",
                     (long long)ts.tv_sec, ts.tv_nsec / NANOSECONDS_TO_MICROSECONDS,
                     (intptr_t)callbackTag, ih, packageId);
@@ -415,7 +418,6 @@ void qat_instance_notification_callbackFn(const CpaInstanceHandle ih, void *call
             break;
     }
 }
-#endif
 
 static int qat_instance_sym_supported(CpaCyCapabilitiesInfo *pCapInfo)
 {
@@ -669,7 +671,6 @@ int qat_hw_init(ENGINE *e)
         qat_instance_details[instNum].qat_instance_started = 1;
         DEBUG("Started Instance No: %d Located on Device: %d\n", instNum, package_id);
 
-#if !defined(__FreeBSD__) && !defined(QAT_DRIVER_INTREE)
         if (enable_sw_fallback) {
             DEBUG("cpaCyInstanceSetNotificationCb instNum = %d\n", instNum);
             status = cpaCyInstanceSetNotificationCb(qat_instance_handles[instNum],
@@ -682,7 +683,6 @@ int qat_hw_init(ENGINE *e)
                 return 0;
             }
         }
-#endif
     }
 
     if (!qat_remap_instances()) {
