@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023-2024 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -68,9 +68,10 @@ typedef struct evp_signature_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
-    int refcnt;
-    void *lock;
-
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
+    CRYPTO_RWLOCK *lock;
+#endif
     OSSL_FUNC_signature_newctx_fn *newctx;
     OSSL_FUNC_signature_sign_init_fn *sign_init;
     OSSL_FUNC_signature_sign_fn *sign;
@@ -325,18 +326,6 @@ int qat_sm2_copy(EVP_PKEY_CTX *dst, EVP_PKEY_CTX *src)
     if (!qat_sm2_init(dst))
         return 0;
 
-    if ((smsctx = OPENSSL_zalloc(sizeof(*smsctx))) == NULL) {
-        WARN("smsctx alloc Failure\n");
-        QATerr(QAT_F_QAT_SM2_COPY, QAT_R_MALLOC_FAILURE);
-        return 0;
-    }
-
-    if ((smdctx = OPENSSL_zalloc(sizeof(*smdctx))) == NULL) {
-        WARN("smdctx alloc Failure\n");
-        QATerr(QAT_F_QAT_SM2_COPY, QAT_R_MALLOC_FAILURE);
-        return 0;
-    }
-
     smsctx = EVP_PKEY_CTX_get_data(src);
     smdctx = EVP_PKEY_CTX_get_data(dst);
 
@@ -970,9 +959,6 @@ int qat_sm2_sign(EVP_PKEY_CTX *ctx,
         BN_CTX_free(bctx);
     }
 
-    BN_free(sig_r);
-    BN_free(sig_s);
-
     if (fallback) {
         WARN("- Fallback to software mode.\n");
         CRYPTO_QAT_LOG("Resubmitting request to SW - %s\n", __func__);
