@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -57,6 +57,7 @@
 #include <openssl/evp.h>
 #include <openssl/rand.h>
 #include <openssl/sha.h>
+#include "e_qat.h"
 #ifdef ENABLE_QAT_HW_GCM
 # include "cpa.h"
 # include "cpa_types.h"
@@ -66,7 +67,6 @@
 # ifdef ENABLE_QAT_SW_GCM
 # include <intel-ipsec-mb.h>
 #endif
-
 #define IV_STATE_UNINITIALISED 0  /* initial state is not initialized */
 #define GCM_TAG_MAX_SIZE       16
 
@@ -79,7 +79,7 @@
 #define GCM_IV_MAX_SIZE     (1024 / 8)
 #define QAT_AES_BLOCK_SIZE   16
 
-typedef struct qat_evp_cipher_st {
+typedef struct qat_evp_aes_cbc_cipher_st {
     int nid;
 
     int block_size;
@@ -116,7 +116,10 @@ typedef struct qat_evp_cipher_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
+    CRYPTO_REF_COUNT references;
+# if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
+# endif
     OSSL_FUNC_cipher_newctx_fn *newctx;
     OSSL_FUNC_cipher_encrypt_init_fn *einit;
     OSSL_FUNC_cipher_decrypt_init_fn *dinit;
@@ -147,7 +150,6 @@ typedef struct qat_gcm_ctx_st {
     unsigned int   tls_aad_set;
     unsigned char* tag;
     unsigned char* calculated_tag;
-    unsigned int   tag_set;
     unsigned int   tag_calculated;
 #endif
 #ifdef ENABLE_QAT_HW_GCM
@@ -188,11 +190,15 @@ typedef struct qat_gcm_ctx_st {
     int key_set;
 
     int qat_svm;
+    int fallback;
 #endif
+    EVP_CIPHER_CTX *sw_ctx;
+    int sw_tls_ctrl;
     int            tls_aad_len;
     int            tag_len;
     int            iv_len;
     unsigned int   iv_set;
+    unsigned int   tag_set;
     int            iv_gen;
     unsigned char* next_iv;
     unsigned char* iv;
@@ -225,7 +231,7 @@ typedef struct qat_aes_gcm_ctx_st {
     QAT_EVP_CIPHER *cipher;
 }QAT_AES_GCM_CTX;
 
-int QAT_AES_GCM_CTX_get_nid(const QAT_AES_GCM_CTX *ctx);
+int qat_aes_gcm_ctx_get_nid(const QAT_AES_GCM_CTX *ctx);
 void qat_gcm_initctx(void *provctx, QAT_GCM_CTX *ctx, size_t keybits,
                      size_t ivlen_min);
 int qat_gcm_get_ctx_params(void *vctx, OSSL_PARAM params[]);
@@ -244,6 +250,7 @@ int qat_gcm_stream_final(void *ctx, unsigned char *out,
 int qat_gcm_cipher(void *ctx, unsigned char *out,
                    size_t *outl, size_t outsize,
                    const unsigned char *in, size_t inl);
+QAT_EVP_CIPHER get_default_cipher_aes_gcm(int nid);
 
 #define QAT_aes_gcm_cipher(alg, lc, UCMODE, flags, kbits, blkbits, ivbits,nid)  \
 static OSSL_FUNC_cipher_get_params_fn alg##_##kbits##_##lc##_get_params;        \
