@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2023 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -117,6 +117,23 @@ static OSSL_FUNC_keyexch_settable_ctx_params_fn qat_keyexch_ecdh_settable_ctx_pa
 static OSSL_FUNC_keyexch_get_ctx_params_fn qat_keyexch_ecdh_get_ctx_params;
 static OSSL_FUNC_keyexch_gettable_ctx_params_fn qat_keyexch_ecdh_gettable_ctx_params;
 
+QAT_EVP_ECDH_KEYEXCH get_default_ecdh_keyexch()
+{
+    static QAT_EVP_ECDH_KEYEXCH s_keyexch;
+    static int initialized = 0;
+    if (!initialized) {
+        QAT_EVP_ECDH_KEYEXCH *keyexch = (QAT_EVP_ECDH_KEYEXCH *)EVP_KEYEXCH_fetch(NULL,"ECDH","provider=default");
+        if (keyexch) {
+           s_keyexch = *keyexch;
+           EVP_KEYEXCH_free((EVP_KEYEXCH *)keyexch);
+           initialized = 1;
+        } else {
+           WARN("EVP_KEYEXCH_fetch from default provider failed");
+        }
+    }
+    return s_keyexch;
+}
+
 static const OSSL_PARAM known_settable_ctx_params[] = {
     OSSL_PARAM_int(OSSL_EXCHANGE_PARAM_EC_ECDH_COFACTOR_MODE, NULL),
     OSSL_PARAM_utf8_string(OSSL_EXCHANGE_PARAM_KDF_TYPE, NULL, 0),
@@ -140,10 +157,13 @@ static const OSSL_PARAM known_gettable_ctx_params[] = {
 static int QAT_ECDH_KEY_up_ref(EC_KEY *r)
 {
     int i;
-
+# if OPENSSL_VERSION_NUMBER < 0x30200000
     if (CRYPTO_UP_REF(&r->references, &i, r->lock) <= 0)
         return 0;
-
+# else
+    if (QAT_CRYPTO_UP_REF(&r->references, &i) <= 0)
+        return 0;
+# endif
     if(i < 2){
         WARN("refcount error");
         return 0;
@@ -157,9 +177,11 @@ static void QAT_ECDH_KEY_free(EC_KEY *r)
 
     if (r == NULL)
         return;
-
+# if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_DOWN_REF(&r->references, &i, r->lock);
-
+# else
+    QAT_CRYPTO_DOWN_REF(&r->references, &i);
+# endif
     if (i > 0)
         return;
 
@@ -175,9 +197,9 @@ static void QAT_ECDH_KEY_free(EC_KEY *r)
         r->group->meth->keyfinish(r);
 
     CRYPTO_free_ex_data(CRYPTO_EX_INDEX_EC_KEY, r, &r->ex_data);
-
+# if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_THREAD_lock_free(r->lock);
-
+# endif
     EC_GROUP_free(r->group);
     EC_POINT_free(r->pub_key);
     BN_clear_free(r->priv_key);
@@ -758,6 +780,15 @@ static int qat_keyexch_ecdh_derive(void *vpecdhctx, unsigned char *secret,
 
     qat_fips_service_indicator = 1;
 #endif
+    if (qat_sw_ecdh_offload != 1 && qat_hw_ecdh_offload != 1) {
+        typedef int (*fun_ptr)(void *, unsigned char *,
+                               size_t *, size_t);
+        fun_ptr fun = get_default_ecdh_keyexch().derive;
+        if (!fun)
+            return 0;
+        return fun(vpecdhctx, secret, psecretlen, outlen);
+    }
+
     switch (pecdhctx->kdf_type) {
     case PROV_ECDH_KDF_NONE:
         ret = qat_keyexch_ecdh_plain_derive(vpecdhctx, secret, psecretlen, outlen);
