@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -50,6 +50,7 @@
 #define __USE_GNU
 
 #include "qat_provider.h"
+#include "qat_prov_hkdf_packet.h"
 #include "qat_prov_ec.h"
 #include "qat_utils.h"
 #include "qat_evp.h"
@@ -76,9 +77,10 @@ typedef struct evp_signature_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
-    int refcnt;
-    void *lock;
-
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
+    CRYPTO_RWLOCK *lock;
+#endif
     OSSL_FUNC_signature_newctx_fn *newctx;
     OSSL_FUNC_signature_sign_init_fn *sign_init;
     OSSL_FUNC_signature_sign_fn *sign;
@@ -333,6 +335,7 @@ static int qat_ecdsa_setup_md(QAT_PROV_ECDSA_CTX *ctx, const char *mdname,
     EVP_MD *md = NULL;
     size_t mdname_len;
     int md_nid, sha1_allowed;
+    qat_WPACKET pkt;
 
     if (mdname == NULL)
         return 1;
@@ -365,7 +368,16 @@ static int qat_ecdsa_setup_md(QAT_PROV_ECDSA_CTX *ctx, const char *mdname,
 
     EVP_MD_CTX_free(ctx->mdctx);
     EVP_MD_free(ctx->md);
+
     ctx->aid_len = 0;
+    if (QAT_WPACKET_init_der(&pkt, ctx->aid_buf, sizeof(ctx->aid_buf))
+        && qat_DER_w_algorithmIdentifier_ECDSA_with_MD(&pkt, -1, ctx->ec,
+                                                        md_nid)
+        && QAT_WPACKET_finish(&pkt)) {
+        QAT_WPACKET_get_total_written(&pkt, &ctx->aid_len);
+        ctx->aid = QAT_WPACKET_get_curr(&pkt);
+    }
+    QAT_WPACKET_cleanup(&pkt);
     ctx->mdctx = NULL;
     ctx->md = md;
     ctx->mdsize = EVP_MD_get_size(ctx->md);
@@ -502,6 +514,13 @@ static int qat_signature_ecdsa_sign(void *vctx, unsigned char *sig, size_t *sigl
         ret = mb_ecdsa_sign(0, tbs, tbslen, sig, &sltmp, ctx->kinv, ctx->r, ctx->ec);
         if (ret <= 0)
             goto end;
+    } else {
+        typedef int (*fun_ptr)(void *, unsigned char *, size_t *,
+                               size_t , const unsigned char *, size_t);
+        fun_ptr fun = get_default_ECDSA_signature().sign;
+        if (!fun)
+            return 0;
+        return fun(vctx, sig, siglen, sigsize, tbs, tbslen);
     }
 #endif
     *siglen = sltmp;
@@ -533,8 +552,16 @@ static int qat_signature_ecdsa_verify(void *vctx, const unsigned char *sig, size
         ret = qat_ecdsa_verify(0, tbs, tbslen, sig, siglen, ctx->ec);
 #endif
 #ifdef ENABLE_QAT_SW_ECDSA
-    if (qat_sw_ecdsa_offload)
+    if (qat_sw_ecdsa_offload) {
         ret = mb_ecdsa_verify(0, tbs, tbslen, sig, siglen, ctx->ec);
+    } else {
+        typedef int (*fun_ptr)(void *, const unsigned char *, size_t,
+                               const unsigned char *, size_t);
+        fun_ptr fun = get_default_ECDSA_signature().verify;
+        if (!fun)
+            return 0;
+        return fun(vctx, sig, siglen, tbs, tbslen);
+    }
 #endif
 
 end:
