@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -70,8 +70,10 @@ typedef void CRYPTO_RWLOCK;
 
 struct kdf_data_st {
     OSSL_LIB_CTX *libctx;
-    CRYPTO_REF_COUNT refcnt;
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
+#endif
 };
 
 typedef struct kdf_data_st QAT_KDF_DATA;
@@ -113,9 +115,10 @@ struct evp_kdf_st {
     int name_id;
     char *type_name;
     const char *description;
-    CRYPTO_REF_COUNT refcnt;
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
-
+#endif
     OSSL_FUNC_kdf_newctx_fn *newctx;
     OSSL_FUNC_kdf_dupctx_fn *dupctx;
     OSSL_FUNC_kdf_freectx_fn *freectx;
