@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2022-2024 Intel Corporation.
+ *   Copyright(c) 2022-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -117,10 +117,10 @@ typedef struct{
     const char *description;
     OSSL_PROVIDER *prov;
 
-    int refcnt;
-# if OPENSSL_VERSION_NUMBER < 0x30200000
-    void *lock;
-# endif
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
+    CRYPTO_RWLOCK *lock;
+#endif
     /* Constructor(s), destructor, information */
     OSSL_FUNC_keymgmt_new_fn *new;
     OSSL_FUNC_keymgmt_free_fn *free;
@@ -147,8 +147,14 @@ typedef struct{
     /* Import and export routines */
     OSSL_FUNC_keymgmt_import_fn *import;
     OSSL_FUNC_keymgmt_import_types_fn *import_types;
+# if OPENSSL_VERSION_NUMBER >= 0x30200000
+    OSSL_FUNC_keymgmt_import_types_ex_fn *import_types_ex;
+# endif
     OSSL_FUNC_keymgmt_export_fn *export;
     OSSL_FUNC_keymgmt_export_types_fn *export_types;
+# if OPENSSL_VERSION_NUMBER >= 0x30200000
+    OSSL_FUNC_keymgmt_export_types_ex_fn *export_types_ex;
+# endif
     OSSL_FUNC_keymgmt_dup_fn *dup;
 
 } QAT_EC_KEYMGMT;
@@ -216,6 +222,8 @@ EC_KEY *qat_ec_key_new(OSSL_LIB_CTX *libctx, const char *propq)
 # endif
     ret->meth = EC_KEY_get_default_method();
 
+    ret->conv_form = POINT_CONVERSION_UNCOMPRESSED;
+
     if (!CRYPTO_new_ex_data(CRYPTO_EX_INDEX_EC_KEY, ret, &ret->ex_data)) {
         goto err;
     }
@@ -285,18 +293,26 @@ static void *qat_keymgmt_ec_gen_init(void *provctx, int selection,
     OSSL_LIB_CTX *libctx = prov_libctx_of(provctx);
     QAT_EC_GEN_CTX *gctx = NULL;
 
-    if (!qat_prov_is_running() || (selection & (OSSL_KEYMGMT_SELECT_ALL)) == 0)
-        return NULL;
+    if (qat_sw_ecdh_offload) {
+        if (!qat_prov_is_running() || (selection & (OSSL_KEYMGMT_SELECT_ALL)) == 0)
+            return NULL;
 
-    if ((gctx = OPENSSL_zalloc(sizeof(*gctx))) != NULL) {
-        gctx->libctx = libctx;
-        gctx->selection = selection;
-        gctx->ecdh_mode = 0;
+        if ((gctx = OPENSSL_zalloc(sizeof(*gctx))) != NULL) {
+             gctx->libctx = libctx;
+             gctx->selection = selection;
+             gctx->ecdh_mode = 0;
 
-        if (!qat_keymgmt_ec_gen_set_params(gctx, params)) {
-            OPENSSL_free(gctx);
-            gctx = NULL;
+            if (!qat_keymgmt_ec_gen_set_params(gctx, params)) {
+                OPENSSL_free(gctx);
+                gctx = NULL;
+            }
         }
+    } else {
+        typedef void * (*fun_ptr)(void *, int, const OSSL_PARAM *);
+        fun_ptr fun = get_default_keymgmt().gen_init;
+        if (!fun)
+            return NULL;
+        return fun(provctx, selection, params);
     }
     return gctx;
 }
@@ -578,11 +594,20 @@ static void *qat_keymgmt_ec_gen(void *genctx, OSSL_CALLBACK *osslcb, void *cbarg
     ret = qat_ec_gen_assign_group(ec, gctx->gen_group);
 
 #if ENABLE_QAT_HW_ECDH
-    ret = ret && qat_ecdh_generate_key(ec);
+    if (qat_hw_ecdh_offload)
+        ret = ret && qat_ecdh_generate_key(ec);
 #endif
 
 #if ENABLE_QAT_SW_ECDH
-    ret = ret && mb_ecdh_generate_key(ec);
+    if (qat_sw_ecdh_offload) {
+        ret = ret && mb_ecdh_generate_key(ec);
+    } else {
+      typedef void * (*fun_ptr)(void *, OSSL_CALLBACK *, void *);
+      fun_ptr fun = get_default_keymgmt().gen;
+      if (!fun)
+          return NULL;
+      return fun(genctx, osslcb, cbarg);
+    }
 #endif
     if (gctx->ecdh_mode != -1)
         ret = ret && qat_ec_set_ecdh_cofactor_mode(ec, gctx->ecdh_mode);
