@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023-2024 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -120,8 +120,10 @@ typedef struct qat_evp_cipher_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
-    CRYPTO_REF_COUNT refcnt;
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
     CRYPTO_RWLOCK *lock;
+#endif
     OSSL_FUNC_cipher_newctx_fn *newctx;
     OSSL_FUNC_cipher_encrypt_init_fn *einit;
     OSSL_FUNC_cipher_decrypt_init_fn *dinit;
@@ -226,6 +228,7 @@ int qat_sm4_ccm_stream_final(void *ctx, unsigned char *out,
 int qat_sm4_ccm_cipher(void *ctx, unsigned char *out,
                        size_t *outl, size_t outsize,
                        const unsigned char *in, size_t inl);
+QAT_EVP_CIPHER qat_get_default_cipher_sm4_ccm();
 
 #  define QAT_sm4_cipher(alg, lc, UCMODE, flags, kbits, blkbits, ivbits,nid)    \
 static OSSL_FUNC_cipher_get_params_fn alg##_##lc##_get_params;                  \
