@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2021-2024 Intel Corporation.
+ *   Copyright(c) 2021-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -69,11 +69,10 @@ typedef struct evp_signature_st {
     char *type_name;
     const char *description;
     OSSL_PROVIDER *prov;
-    int refcnt;
-# if OPENSSL_VERSION_NUMBER < 0x30200000
-    void *lock;
-# endif
-
+    CRYPTO_REF_COUNT references;
+#if OPENSSL_VERSION_NUMBER < 0x30200000
+    CRYPTO_RWLOCK *lock;
+#endif
     OSSL_FUNC_signature_newctx_fn *newctx;
     OSSL_FUNC_signature_sign_init_fn *sign_init;
     OSSL_FUNC_signature_sign_fn *sign;
@@ -699,12 +698,10 @@ int mb_ecdsa_sm2_sign(EVP_MD_CTX *mctx,
     }
 # endif
 
-# if defined(QAT_OPENSSL_3) && !defined(QAT_OPENSSL_PROVIDER)
-    if (qat_openssl3_sm2_fallback == 1) {
+    if ((qat_openssl3_sm2_fallback == 1) || (!qat_sw_sm2_offload)) {
         DEBUG("- Switched to software mode\n");
         goto use_sw_method;
     }
-# endif
 
     /* QAT SW initialization fail, switching to OpenSSL. */
     if (fallback_to_openssl)
@@ -1017,12 +1014,10 @@ int mb_ecdsa_sm2_verify(EVP_MD_CTX *mctx,
     }
 # endif
 
-# if defined(QAT_OPENSSL_3) && !defined(QAT_OPENSSL_PROVIDER)
-    if (qat_openssl3_sm2_fallback == 1) {
+    if ((qat_openssl3_sm2_fallback == 1) || (!qat_sw_sm2_offload)) {
         DEBUG("- Switched to software mode\n");
         goto use_sw_method;
     }
-# endif
 
     /* QAT SW initialization fail, switching to OpenSSL. */
     if (fallback_to_openssl)
