@@ -3,7 +3,7 @@
  *
  *   BSD LICENSE
  *
- *   Copyright(c) 2023 Intel Corporation.
+ *   Copyright(c) 2023-2025 Intel Corporation.
  *   All rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or without
@@ -88,7 +88,9 @@ static const unsigned char iv[] = {
 static int run_aesccm256_tls(void *args)
 {
     TEST_PARAMS *temp_args = (TEST_PARAMS *) args;
+#ifndef QAT_OPENSSL_PROVIDER
     ENGINE *e = temp_args->e;
+#endif
     int size = INPUT_TEXT_SIZE;
     int print_output = temp_args->print_output;
     int verify = temp_args->verify;
@@ -129,6 +131,7 @@ static int run_aesccm256_tls(void *args)
     EVP_CIPHER_CTX *ctx = NULL;
     EVP_CIPHER_CTX *dec_ctx = NULL;
 
+#ifndef QAT_OPENSSL_PROVIDER
     if (e != NULL) {
         const EVP_CIPHER *c = ENGINE_get_cipher(e, NID_aes_256_ccm);
 
@@ -137,6 +140,7 @@ static int run_aesccm256_tls(void *args)
             e = NULL;
         }
     }
+#endif
 
     if (input == NULL) {
         INFO("# FAIL: [%s] --- Initial parameters malloc failed ! \n",
@@ -167,7 +171,11 @@ static int run_aesccm256_tls(void *args)
      * IV used in the initialization of the cipher: in TLS case it is set
      * using ctrl() instead of EncryptInit_ex().
      */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit_ex(ctx, EVP_aes_256_ccm(), NULL, NULL, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_256_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -192,7 +200,11 @@ static int run_aesccm256_tls(void *args)
     }
 
     /* set 32-byte of key to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit_ex(ctx, NULL, NULL, key, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_256_ccm(), e, key, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed for setting key: ret %d\n", __func__, ret);
         goto err;
@@ -257,7 +269,11 @@ static int run_aesccm256_tls(void *args)
     }
 
     /* Initialise the context for decryption with 32-byte key and 12-byte IV. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_256_ccm(), NULL, NULL, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_256_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -282,7 +298,11 @@ static int run_aesccm256_tls(void *args)
     }
 
     /* set 32-byte of key to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit_ex(dec_ctx, NULL, NULL, key, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_256_ccm(), e, key, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed for setting key: ret %d\n", __func__, ret);
         goto err;
@@ -370,7 +390,9 @@ static int run_aesccm256_tls(void *args)
 static int run_aesccm256_update(void *args)
 {
     TEST_PARAMS *temp_args = (TEST_PARAMS *) args;
+#ifndef QAT_OPENSSL_PROVIDER
     ENGINE *e = temp_args->e;
+#endif
     int size = INPUT_TEXT_SIZE;
     int print_output = temp_args->print_output;
     int verify = temp_args->verify;
@@ -388,6 +410,7 @@ static int run_aesccm256_update(void *args)
     EVP_CIPHER_CTX *ctx = NULL;
     EVP_CIPHER_CTX *dec_ctx = NULL;
 
+#ifndef QAT_OPENSSL_PROVIDER
     if (e != NULL) {
         const EVP_CIPHER *c = ENGINE_get_cipher(e, NID_aes_256_ccm);
 
@@ -396,6 +419,7 @@ static int run_aesccm256_update(void *args)
             e = NULL;
         }
     }
+#endif
 
     /*
      * Set the plaintext input data.
@@ -422,7 +446,11 @@ static int run_aesccm256_update(void *args)
     }
 
     /* Initialize encryption context for aes-ccm */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit(ctx, EVP_aes_256_ccm(), NULL, NULL);
+#else
     ret = EVP_EncryptInit_ex(ctx, EVP_aes_256_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() encryption failed while setting context: ret %d\n", __func__, ret);
         goto err;
@@ -443,15 +471,21 @@ static int run_aesccm256_update(void *args)
         goto err;
     }
 
+#ifndef QAT_OPENSSL_PROVIDER
     /* set 32-byte of key to the aes-ccm context. */
     ret = EVP_EncryptInit_ex(ctx, NULL, NULL, key, NULL);
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed while setting key: ret %d\n", __func__, ret);
         goto err;
     }
+#endif
 
     /* set 12-byte of iv to the aes-ccm context. */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_EncryptInit(ctx, NULL, key, iv);
+#else
     ret = EVP_EncryptInit_ex(ctx, NULL, NULL, NULL, iv);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_EncryptInit_ex() failed while setting iv: ret %d\n", __func__, ret);
         goto err;
@@ -512,7 +546,11 @@ static int run_aesccm256_update(void *args)
     }
 
     /* Initialize decryption context for aes-ccm */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit(dec_ctx, EVP_aes_256_ccm(), NULL, NULL);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, EVP_aes_256_ccm(), e, NULL, NULL);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
@@ -543,7 +581,11 @@ static int run_aesccm256_update(void *args)
      *  Initialise the aes-ccm decryption context with 32-byte key and
      *  12-byte IV.
      */
+#ifdef QAT_OPENSSL_PROVIDER
+    ret = EVP_DecryptInit(dec_ctx, NULL, key, iv);
+#else
     ret = EVP_DecryptInit_ex(dec_ctx, NULL, NULL, key, iv);
+#endif
     if (ret != 1) {
         INFO("# FAIL: [%s] --- EVP_DecryptInit_ex() failed: ret %d\n",
              __func__, ret);
